use stanza::client::Stanza;
use tokio::sync::oneshot::{self};

#[derive(Debug)]
pub enum Error {
    AlreadyConnected,
    // TODO: change to Connecting(ConnectingError)
    Connection(ConnectionError),
    Presence(Reason),
    Roster(Reason),
    SendMessage(Reason),
    AlreadyDisconnected,
    LostConnection,
}

#[derive(Debug)]
pub enum ConnectionError {
    ConnectionFailed(Reason),
    RosterRetreival(Reason),
    SendPresence(Reason),
    NoCachedStatus(Reason),
}

pub struct RosterError(pub Reason);

impl From<RosterError> for Error {
    fn from(e: RosterError) -> Self {
        Self::Roster(e.0)
    }
}

impl From<RosterError> for ConnectionError {
    fn from(e: RosterError) -> Self {
        Self::RosterRetreival(e.0)
    }
}

pub struct StatusError(Reason);

impl From<StatusError> for Error {
    fn from(e: StatusError) -> Self {
        Error::Presence(e.0)
    }
}

impl From<StatusError> for ConnectionError {
    fn from(e: StatusError) -> Self {
        Self::SendPresence(e.0)
    }
}

#[derive(Debug)]
pub enum Reason {
    // TODO: organisastion of error into internal error thing
    Timeout,
    Stream(stanza::stream_error::Error),
    Stanza(stanza::stanza_error::Error),
    Jabber(jabber::Error),
    XML(peanuts::Error),
    SQL(sqlx::Error),
    // JID(jid::ParseError),
    LostConnection,
    OneshotRecv(oneshot::error::RecvError),
    UnexpectedStanza(Stanza),
}

impl From<oneshot::error::RecvError> for Reason {
    fn from(e: oneshot::error::RecvError) -> Reason {
        Self::OneshotRecv(e)
    }
}

impl From<peanuts::Error> for Reason {
    fn from(e: peanuts::Error) -> Self {
        Self::XML(e)
    }
}

// impl From<jid::ParseError> for Reason {
//     fn from(e: jid::ParseError) -> Self {
//         Self::JID(e)
//     }
// }

impl From<sqlx::Error> for Reason {
    fn from(e: sqlx::Error) -> Self {
        Self::SQL(e)
    }
}

impl From<jabber::Error> for Reason {
    fn from(e: jabber::Error) -> Self {
        Self::Jabber(e)
    }
}
