use sqlx::Sqlite;

#[derive(Debug, Default, sqlx::FromRow, Clone)]
pub struct Online {
    pub show: Option<Show>,
    pub status: Option<String>,
    #[sqlx(skip)]
    priority: Option<i8>,
}

#[derive(Debug, Clone, Copy)]
pub enum Show {
    Away,
    Chat,
    DoNotDisturb,
    ExtendedAway,
}

impl sqlx::Type<Sqlite> for Show {
    fn type_info() -> <Sqlite as sqlx::Database>::TypeInfo {
        <&str as sqlx::Type<Sqlite>>::type_info()
    }
}

impl sqlx::Decode<'_, Sqlite> for Show {
    fn decode(
        value: <Sqlite as sqlx::Database>::ValueRef<'_>,
    ) -> Result<Self, sqlx::error::BoxDynError> {
        let value = <&str as sqlx::Decode<Sqlite>>::decode(value)?;
        match value {
            "away" => Ok(Self::Away),
            "chat" => Ok(Self::Chat),
            "do-not-disturb" => Ok(Self::DoNotDisturb),
            "extended-away" => Ok(Self::ExtendedAway),
            _ => unreachable!(),
        }
    }
}

impl sqlx::Encode<'_, Sqlite> for Show {
    fn encode_by_ref(
        &self,
        buf: &mut <Sqlite as sqlx::Database>::ArgumentBuffer<'_>,
    ) -> Result<sqlx::encode::IsNull, sqlx::error::BoxDynError> {
        let value = match self {
            Show::Away => "away",
            Show::Chat => "chat",
            Show::DoNotDisturb => "do-not-disturb",
            Show::ExtendedAway => "extended-away",
        };
        <&str as sqlx::Encode<Sqlite>>::encode(value, buf)
    }
}

#[derive(Debug, Default)]
pub struct Offline {
    status: Option<String>,
}

#[derive(Debug)]
pub enum Presence {
    Online(Online),
    Offline(Offline),
}
